@@ -34,7 +34,7 @@ BEGIN {
 }
 
 use Mail::SpamAssassin;
-use Mail::SpamAssassin::Util qw(untaint_var);
+use Mail::SpamAssassin::Util qw(untaint_var am_running_on_windows);
 use Mail::SpamAssassin::BayesStore;
 use Mail::SpamAssassin::Logger;
 
@@ -458,7 +458,7 @@ sub _upgrade_db {
     # it to be interrupted.
     local $SIG{'INT'} = 'IGNORE';
     local $SIG{'TERM'} = 'IGNORE';
-    local $SIG{'HUP'} = 'IGNORE' if (!Mail::SpamAssassin::Util::am_running_on_windows());
+    local $SIG{'HUP'} = 'IGNORE' if !am_running_on_windows();
 
     # older versions used scancount, so kill the stupid little file ...
     my $msgc = $path.'_msgcount';
@@ -553,7 +553,7 @@ sub _upgrade_db {
     # it to be interrupted.
     local $SIG{'INT'} = 'IGNORE';
     local $SIG{'TERM'} = 'IGNORE';
-    local $SIG{'HUP'} = 'IGNORE' if (!Mail::SpamAssassin::Util::am_running_on_windows());
+    local $SIG{'HUP'} = 'IGNORE' if !am_running_on_windows();
 
     # now rename in the new one.  Try several extensions
     for my $ext ($self->DB_EXTENSIONS) {
@@ -766,7 +766,7 @@ sub token_expiration {
     {
       local $SIG{'INT'} = 'IGNORE';
       local $SIG{'TERM'} = 'IGNORE';
-      local $SIG{'HUP'} = 'IGNORE' if (!Mail::SpamAssassin::Util::am_running_on_windows());
+      local $SIG{'HUP'} = 'IGNORE' if !am_running_on_windows();
 
       # now rename in the new one.  Try several extensions
       for my $ext ($self->DB_EXTENSIONS) {
@@ -1258,7 +1258,7 @@ sub _sync_journal_trapped {
   {
     local $SIG{'INT'} = 'IGNORE';
     local $SIG{'TERM'} = 'IGNORE';
-    local $SIG{'HUP'} = 'IGNORE' if (!Mail::SpamAssassin::Util::am_running_on_windows());
+    local $SIG{'HUP'} = 'IGNORE' if !am_running_on_windows();
 
     # retire the journal, so we can update the db files from it in peace.
     # TODO: use locking here
@@ -1433,11 +1433,13 @@ sub perform_upgrade {
     local $SIG{'__DIE__'};	# do not run user die() traps in here
 
     use File::Basename;
-    use File::Copy;
 
     # bayes directory
     my $main = $self->{bayes}->{main};
     my $path = $main->sed_path($main->{conf}->{bayes_path});
+
+    # prevent dirname() from tainting the result, it assumes $1 is not tainted
+    local($1,$2,$3);  # Bug 6310
     my $dir = dirname($path);
 
     # make temporary copy since old dbm and new dbm may have same name
@@ -1454,7 +1456,10 @@ sub perform_upgrade {
     for (@files) {
       my $src = "$dir/$_";
       my $dst = "$dir/old_$_";
-      copy($src, $dst) || die "bayes: can't copy $src to $dst: $!\n";
+      eval q{
+        use File::Copy;
+        copy($src, $dst);
+      } || die "bayes: can't copy $src to $dst: $!\n";
     }
 
     # delete previous to make way for import
@@ -1825,7 +1830,7 @@ sub restore_database {
 
   local $SIG{'INT'} = 'IGNORE';
   local $SIG{'TERM'} = 'IGNORE';
-  local $SIG{'HUP'} = 'IGNORE' if (!Mail::SpamAssassin::Util::am_running_on_windows());
+  local $SIG{'HUP'} = 'IGNORE' if !am_running_on_windows();
 
   untie %new_toks;
   untie %new_seen;
