@@ -39,8 +39,6 @@ The results are incorporated into SpamAssassin as the BAYES_* rules.
 
 =head1 METHODS
 
-=over 4
-
 =cut
 
 package Mail::SpamAssassin::Plugin::Bayes;
@@ -200,8 +198,8 @@ use constant ADD_INVIZ_TOKENS_NO_PREFIX => 0;
   'Organization'	=> '*o',
   'Organisation'        => '*o',
   'Content-Type'	=> '*c',
-  'X-Spam-Relays-Trusted' => '*RT',
-  'X-Spam-Relays-Untrusted' => '*RU',
+  'x-spam-relays-trusted' => '*RT',
+  'x-spam-relays-untrusted' => '*RU',
 );
 
 # How many seconds should the opportunistic_expire lock be valid?
@@ -832,7 +830,7 @@ skip:
             });
 
   $permsgstatus->set_tag ('TOKENSUMMARY', sub {
-              if( defined $self->{tag_data}{BAYESTC} )
+              if ( defined $permsgstatus->{tag_data}{BAYESTC} )
                 {
                   my $tcount_neutral = $permsgstatus->{tag_data}{BAYESTCLEARNED}
                                     - $permsgstatus->{tag_data}{BAYESTCSPAMMY}
@@ -1563,7 +1561,3 @@ sub bayes_report_make_list {
 }
 
 1;
-
-=back
-
-=cut
