@@ -1,6 +1,6 @@
 =head1 NAME
 
-Mail::SpamAssassin::Plugin::Check
+Mail::SpamAssassin::Plugin::Check - primary message check functionality
 
 =head1 SYNOPSIS
 
@@ -57,10 +57,13 @@ sub check_main {
     dbg("check: adding caller rule hits, %d rules", scalar(@caller_rule_hits));
     for my $caller_rule_hit (@caller_rule_hits) {
       next if ref $caller_rule_hit ne 'HASH';
-      my($rulename, $area, $score, $value, $ruletype, $tflags, $description) =
-        @$caller_rule_hit{qw(rule area score value ruletype tflags descr)};
+      my($rulename, $area, $score, $defscore, $value,
+         $ruletype, $tflags, $description) =
+        @$caller_rule_hit{qw(rule area score defscore value
+                             ruletype tflags descr)};
       $pms->got_hit($rulename, $area,
                     !defined $score ? () : (score => $score),
+                    !defined $defscore ? () : (defscore => $defscore),
                     !defined $value ? () : (value => $value),
                     !defined $tflags ? () : (tflags => $tflags),
                     !defined $description ? () : (description => $description),
@@ -198,7 +201,7 @@ sub check_main {
   }
 
   if ($pms->{deadline_exceeded}) {
-    $pms->got_hit('TIME_LIMIT_EXCEEDED', '', score => 0.001,
+    $pms->got_hit('TIME_LIMIT_EXCEEDED', '', defscore => 0.001,
                   description => 'Exceeded time limit / deadline');
   }
 
@@ -307,64 +310,51 @@ sub run_generic_tests {
   my $package_name = __PACKAGE__;
   my $methodname = $package_name."::_".$ruletype."_tests_".$clean_priority;
 
-  if (defined &{$methodname} && !$doing_user_rules) {
-run_compiled_method:
-  # dbg("rules: run_generic_tests - calling %s", $methodname);
-    my $t = Mail::SpamAssassin::Timeout->new({ deadline => $master_deadline });
-    my $err = $t->run(sub {
-      no strict "refs";
-      $methodname->($pms, @{$opts{args}});
-    });
-    if ($t->timed_out() && $master_deadline && time > $master_deadline) {
-      info("check: exceeded time limit in $methodname, skipping further tests");
-      $pms->{deadline_exceeded} = 1;
+  if (!defined &{$methodname} || $doing_user_rules) {
+
+    # use %nopts for named parameter-passing; it's more friendly
+    # to future-proof subclassing, since new parameters can be added without
+    # breaking third-party subclassed implementations of this plugin.
+    my %nopts = (
+      ruletype => $ruletype,
+      doing_user_rules => $doing_user_rules,
+      priority => $priority,
+      clean_priority => $clean_priority
+    );
+
+    # build up the eval string...
+    $self->{evalstr_methodname} = $methodname;
+    $self->{evalstr_chunk_current_methodname} = undef;
+    $self->{evalstr_chunk_methodnames} = [];
+    $self->{evalstr_chunk_prefix} = []; # stack (array) of source code sections
+    $self->{evalstr} = ''; $self->{evalstr_l} = 0;
+    $self->{evalstr2} = '';
+    $self->begin_evalstr_chunk($pms);
+
+    $self->push_evalstr_prefix($pms, '
+        # start_rules_plugin_code '.$ruletype.' '.$priority.'
+        my $scoresptr = $self->{conf}->{scores};
+    ');
+    if (defined $opts{pre_loop_body}) {
+      $opts{pre_loop_body}->($self, $pms, $conf, %nopts);
+    }
+    $self->add_evalstr($pms,
+                       $self->start_rules_plugin_code($ruletype, $priority) );
+    while (my($rulename, $test) = each %{$opts{testhash}->{$priority}}) {
+      $opts{loop_body}->($self, $pms, $conf, $rulename, $test, %nopts);
+    }
+    if (defined $opts{post_loop_body}) {
+      $opts{post_loop_body}->($self, $pms, $conf, %nopts);
     }
-    return;
-  }
 
-  # use %nopts for named parameter-passing; it's more friendly to future-proof
-  # subclassing, since new parameters can be added without breaking third-party
-  # subclassed implementations of this plugin.
-  my %nopts = (
-    ruletype => $ruletype,
-    doing_user_rules => $doing_user_rules,
-    priority => $priority,
-    clean_priority => $clean_priority
-  );
+    $self->flush_evalstr($pms, 'run_generic_tests');
+    $self->free_ruleset_source($pms, $ruletype, $priority);
 
-  # build up the eval string...
-  $self->{evalstr_methodname} = $methodname;
-  $self->{evalstr_chunk_current_methodname} = undef;
-  $self->{evalstr_chunk_methodnames} = [];
-  $self->{evalstr_chunk_prefix} = [];  # stack (array) of source code sections
-  $self->{evalstr} = ''; $self->{evalstr_l} = 0;
-  $self->{evalstr2} = '';
-  $self->begin_evalstr_chunk($pms);
+    # clear out a previous version of this method
+    undef &{$methodname};
 
-  $self->push_evalstr_prefix($pms, '
-      # start_rules_plugin_code '.$ruletype.' '.$priority.'
-      my $scoresptr = $self->{conf}->{scores};
-  ');
-  if (defined $opts{pre_loop_body}) {
-    $opts{pre_loop_body}->($self, $pms, $conf, %nopts);
-  }
-  $self->add_evalstr($pms,
-                     $self->start_rules_plugin_code($ruletype, $priority) );
-  while (my($rulename, $test) = each %{$opts{testhash}->{$priority}}) {
-    $opts{loop_body}->($self, $pms, $conf, $rulename, $test, %nopts);
-  }
-  if (defined $opts{post_loop_body}) {
-    $opts{post_loop_body}->($self, $pms, $conf, %nopts);
-  }
-
-  $self->flush_evalstr($pms, 'run_generic_tests');
-  $self->free_ruleset_source($pms, $ruletype, $priority);
-
-  # clear out a previous version of this method
-  undef &{$methodname};
-
-  # generate the loop that goes through each line...
-  my $evalstr = <<"EOT";
+    # generate the loop that goes through each line...
+    my $evalstr = <<"EOT";
   {
     package $package_name;
 
@@ -373,40 +363,51 @@ run_compiled_method:
     sub $methodname {
 EOT
 
-  for my $chunk_methodname (@{$self->{evalstr_chunk_methodnames}}) {
-    $evalstr .= "      $chunk_methodname(\@_);\n";
-  }
+    for my $chunk_methodname (@{$self->{evalstr_chunk_methodnames}}) {
+      $evalstr .= "      $chunk_methodname(\@_);\n";
+    }
 
-  $evalstr .= <<"EOT";
+    $evalstr .= <<"EOT";
     }
 
     1;
   }
 EOT
 
-  delete $self->{evalstr};   # free up some RAM before we eval()
-  delete $self->{evalstr2};
-  delete $self->{evalstr_methodname};
-  delete $self->{evalstr_chunk_current_methodname};
-  delete $self->{evalstr_chunk_methodnames};
-  delete $self->{evalstr_chunk_prefix};
+    delete $self->{evalstr};   # free up some RAM before we eval()
+    delete $self->{evalstr2};
+    delete $self->{evalstr_methodname};
+    delete $self->{evalstr_chunk_current_methodname};
+    delete $self->{evalstr_chunk_methodnames};
+    delete $self->{evalstr_chunk_prefix};
 
-  dbg("rules: run_generic_tests - compiling eval code: %s, priority %s",
-      $ruletype, $priority);
-# dbg("rules: eval code to compile: $evalstr");
-  my $eval_result;
-  { my $timer = $self->{main}->time_method('compile_gen');
-    $eval_result = eval($evalstr);
-  }
-  if (!$eval_result) {
-    my $eval_stat = $@ ne '' ? $@ : "errno=$!";  chomp $eval_stat;
-    warn "rules: failed to compile $ruletype tests, skipping:\n".
-         "\t($eval_stat)\n";
-    $pms->{rule_errors}++;
-  }
-  else {
+    dbg("rules: run_generic_tests - compiling eval code: %s, priority %s",
+        $ruletype, $priority);
+  # dbg("rules: eval code to compile: $evalstr");
+    my $eval_result;
+    { my $timer = $self->{main}->time_method('compile_gen');
+      $eval_result = eval($evalstr);
+    }
+    if (!$eval_result) {
+      my $eval_stat = $@ ne '' ? $@ : "errno=$!";  chomp $eval_stat;
+      warn "rules: failed to compile $ruletype tests, skipping:\n".
+           "\t($eval_stat)\n";
+      $pms->{rule_errors}++;
+      return;
+    }
     dbg("rules: compiled $ruletype tests");
-    goto run_compiled_method;
+  }
+
+#run_compiled_method:
+# dbg("rules: run_generic_tests - calling %s", $methodname);
+  my $t = Mail::SpamAssassin::Timeout->new({ deadline => $master_deadline });
+  my $err = $t->run(sub {
+    no strict "refs";
+    $methodname->($pms, @{$opts{args}});
+  });
+  if ($t->timed_out() && $master_deadline && time > $master_deadline) {
+    info("check: exceeded time limit in $methodname, skipping further tests");
+    $pms->{deadline_exceeded} = 1;
   }
 }
 
