@@ -24,6 +24,7 @@ Mail::SpamAssassin::Plugin::DKIM - perform DKIM verification tests
  loadplugin Mail::SpamAssassin::Plugin::DKIM [/path/to/DKIM.pm]
 
 Taking into account signatures from any signing domains:
+
  full   DKIM_SIGNED           eval:check_dkim_signed()
  full   DKIM_VALID            eval:check_dkim_valid()
  full   DKIM_VALID_AU         eval:check_dkim_valid_author_sig()
@@ -31,6 +32,7 @@ Taking into account signatures from any signing domains:
 Taking into account signatures from specified signing domains only:
 (quotes may be omitted on domain names consisting only of letters, digits,
 dots, and minus characters)
+
  full   DKIM_SIGNED_MY1       eval:check_dkim_signed('dom1','dom2',...)
  full   DKIM_VALID_MY1        eval:check_dkim_valid('dom1','dom2',...)
  full   DKIM_VALID_AU_MY1     eval:check_dkim_valid_author_sig('d1','d2',...)
@@ -38,6 +40,7 @@ dots, and minus characters)
  full   __DKIM_DEPENDABLE     eval:check_dkim_dependable()
 
 Author Domain Signing Practices (ADSP) from any author domains:
+
  header DKIM_ADSP_NXDOMAIN    eval:check_dkim_adsp('N')
  header DKIM_ADSP_ALL         eval:check_dkim_adsp('A')
  header DKIM_ADSP_DISCARD     eval:check_dkim_adsp('D')
@@ -46,6 +49,7 @@ Author Domain Signing Practices (ADSP) from any author domains:
  header DKIM_ADSP_CUSTOM_HIGH eval:check_dkim_adsp('3')
 
 Author Domain Signing Practices (ADSP) from specified author domains only:
+
  header DKIM_ADSP_MY1         eval:check_dkim_adsp('*','dom1','dom2',...)
 
  describe DKIM_SIGNED   Message has a DKIM or DK signature, not necessarily valid
@@ -61,6 +65,7 @@ Author Domain Signing Practices (ADSP) from specified author domains only:
  describe DKIM_ADSP_CUSTOM_HIGH adsp_override is CUSTOM_HIGH, no valid author domain signature
 
 For compatibility with pre-3.3.0 versions, the following are synonyms:
+
  OLD: eval:check_dkim_verified = NEW: eval:check_dkim_valid
  OLD: eval:check_dkim_signall  = NEW: eval:check_dkim_adsp('A')
  OLD: eval:check_dkim_signsome = NEW: redundant, semantically always true
@@ -208,6 +213,20 @@ Same as C<whitelist_from_dkim>, but used for the default whitelist entries
 in the SpamAssassin distribution.  The whitelist score is lower, because
 these are often targets for abuse of public mailers which sign their mail.
 
+=item unwhitelist_from_dkim author@example.com [signing-domain]
+
+Removes an email address with its corresponding signing-domain field
+from def_whitelist_from_dkim and whitelist_from_dkim tables, if it exists.
+Parameters to unwhitelist_from_dkim must exactly match the parameters of
+a corresponding whitelist_from_dkim or def_whitelist_from_dkim config
+option which created the entry, for it to be removed (a domain name is
+matched case-insensitively);  i.e. if a signing-domain parameter was
+specified in a whitelisting command, it must also be specified in the
+unwhitelisting command.
+
+Useful for removing undesired default entries from a distributed configuration
+by a local or site-specific configuration or by C<user_prefs>.
+
 =item adsp_override domain [signing-practices]
 
 Currently few domains publish their signing practices (RFC 5617 - ADSP),
@@ -357,8 +376,9 @@ Example:
       }
       my $address = $1;
       my $sdid = defined $2 ? $2 : '';  # empty implies author domain signature
-      $self->{parser}->add_to_addrlist_rcvd('whitelist_from_dkim',
-                                            $address, $sdid);
+      $address =~ s/(\@[^@]*)\z/lc($1)/e;  # lowercase the email address domain
+      $self->{parser}->add_to_addrlist_dkim('whitelist_from_dkim',
+                                            $address, lc $sdid);
     }
   });
 
@@ -376,8 +396,31 @@ Example:
       }
       my $address = $1;
       my $sdid = defined $2 ? $2 : '';  # empty implies author domain signature
-      $self->{parser}->add_to_addrlist_rcvd('def_whitelist_from_dkim',
-                                            $address, $sdid);
+      $address =~ s/(\@[^@]*)\z/lc($1)/e;  # lowercase the email address domain
+      $self->{parser}->add_to_addrlist_dkim('def_whitelist_from_dkim',
+                                            $address, lc $sdid);
+    }
+  });
+
+  push (@cmds, {
+    setting => 'unwhitelist_from_dkim',
+    type => $Mail::SpamAssassin::Conf::CONF_TYPE_ADDRLIST,
+    code => sub {
+      my ($self, $key, $value, $line) = @_;
+      local ($1,$2);
+      unless (defined $value && $value !~ /^$/) {
+        return $Mail::SpamAssassin::Conf::MISSING_REQUIRED_VALUE;
+      }
+      unless ($value =~ /^(\S+)(?:\s+(\S+))?$/) {
+        return $Mail::SpamAssassin::Conf::INVALID_VALUE;
+      }
+      my $address = $1;
+      my $sdid = defined $2 ? $2 : '';  # empty implies author domain signature
+      $address =~ s/(\@[^@]*)\z/lc($1)/e;  # lowercase the email address domain
+      $self->{parser}->remove_from_addrlist_dkim('whitelist_from_dkim',
+                                                 $address, lc $sdid);
+      $self->{parser}->remove_from_addrlist_dkim('def_whitelist_from_dkim',
+                                                 $address, lc $sdid);
     }
   });
 
@@ -418,6 +461,8 @@ Example:
 How many seconds to wait for a DKIM query to complete, before
 scanning continues without the DKIM result.
 
+=back
+
 =cut
 
   push (@cmds, {
@@ -494,10 +539,12 @@ sub check_dkim_adsp {
     # don't bother
   } else {
     $self->_check_dkim_adsp($pms)  if !$pms->{dkim_checked_adsp};
+
     # an asterisk indicates any ADSP type can match (as long as
     # there is no valid author domain signature present)
-    if ($adsp_char ne '*' &&
-        !(grep { $_ eq $adsp_char} values %{$pms->{dkim_adsp}}) ) {
+    $adsp_char = 'NAD123'  if $adsp_char eq '*';  # a shorthand for NAD123
+
+    if ( !(grep { index($adsp_char,$_) >= 0 } values %{$pms->{dkim_adsp}}) ) {
       # not the right ADSP type
     } elsif (!@domains_list) {
       $result = 1;  # no additional constraints, any author domain will do
@@ -776,8 +823,10 @@ sub _check_dkim_signature {
         }
       }
       if (would_log("dbg","dkim")) {
-        dbg("dkim: i=%s, d=%s, a=%s, c=%s, %s%s, %s",
-          $signature->identity, $d,
+        dbg("dkim: %s, i=%s, d=%s, s=%s, a=%s, c=%s, %s%s, %s",
+          map { !defined $_ ? '(undef)' : $_ }
+          $signature->isa('Mail::DKIM::DkSignature') ? 'DK' : 'DKIM',
+          $signature->identity, $d, $signature->selector,
           $signature->algorithm, scalar($signature->canonicalization),
           ($sig_result_supported ? $signature : $verifier)->result,
           !$expired ? '' : ', expired',
