@@ -17,6 +17,16 @@
 # limitations under the License.
 # </@LICENSE>
 
+=head1 NAME
+
+Mail:SpamAssassin::Util::DependencyInfo - spamassassin debugging helpers
+
+=head1 SYNOPSIS
+
+loadplugin Mail:SpamAssassin::Util::DependencyInfo
+
+=cut
+
 package Mail::SpamAssassin::Util::DependencyInfo;
 
 use strict;
@@ -218,11 +228,17 @@ $have_sha ? {
 
 ###########################################################################
 
+=head1 METHODS
+
+=over 4
+
 =item $f->debug_diagnostics ()
 
 Output some diagnostic information, useful for debugging SpamAssassin
 problems.
 
+=back
+
 =cut
 
 sub debug_diagnostics {
